/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: Property.java,v e9b6239881c9 2010/07/11 13:55:48 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.semantic;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import it.tidalwave.util.Id;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public final class Property<T> implements Comparable<Property<T>>
  {
    @Nonnull
    private final Id id;

    @Nonnull
    private final T value;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public Property (final @Nonnull String id, final @Nonnull T value)
      {
        this(new Id(id), value);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public Property (final @Nonnull Id id, final @Nonnull T value)
      {
        this.id = id;
        this.value = value;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Id getId()
      {
        return id;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public T getValue()
      {
        return value;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public int compareTo (final @Nonnull Property<T> other)
      {
        return this.id.compareTo(other.id);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public boolean equals (final @CheckForNull Object other)
      {
        return (other != null)
               && (other instanceof Property)
               && this.id.equals(((Property)other).id)
               && this.value.equals(((Property)other).value);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public int hashCode()
      {
        return id.hashCode() * 39 + value.hashCode();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public String toString()
      {
        return String.format("Property[%s=%s]", id, value);
      }
  }
